extends CanvasItem
class_name ImagePuller

enum Side { Left, Right }

@export var side : ImagePuller.Side = Side.Left
@export var tracking : UltraleapDevice
@export var distortion_texture : Texture2D
@export var raw_image : Texture2D

var data_img : Image = Image.new()
var data : PackedByteArray
var last_matrix_version = 999
var device_index : int = 0

func _ready():
	data_img.set_data(1, 1, false, Image.FORMAT_L8, PackedByteArray([0]))
	get_viewport().size_changed.connect(_on_size_changed)

func _process(_delta):
	if not Engine.is_editor_hint():
		if tracking == null:
			return

		var image : UltraleapImage
		
		# Get the image. If in view, left lens pointing you is the right lens for the camera
		if tracking.get_tracking_mode() == UltraleapTypes.TrackingMode.Desktop:
			image = tracking.get_right_image() if side == Side.Left else tracking.get_left_image()
		else:
			image = tracking.get_right_image() if side == Side.Right else tracking.get_left_image()
			
		if image == null:
			return
		
		# Get image size
		var image_size : Vector2i = image.get_size()
		data = image.get_data()

		# Process the new distortion matrix if there's a new one
		if image.get_matrix_version() != last_matrix_version:
			var distortion_matrix : PackedVector2Array = image.get_distortion_matrix()
			if distortion_matrix.size() == 64 * 64:
				process_distortion_matrix(distortion_matrix)
				last_matrix_version = image.get_matrix_version()

		if data.size() > 0:
			data_img.set_data(image_size.x, image_size.y, false, Image.FORMAT_L8, data)

		if raw_image != null:
			raw_image.set_image(data_img)


func process_distortion_matrix(matrix : PackedVector2Array):
	var distortion_img : Image = Image.new()
	distortion_img.set_data(64, 64, false, Image.FORMAT_RGF, matrix.to_byte_array())
	distortion_texture.set_image(distortion_img)


func _on_size_changed():
	var size : Vector2i = get_viewport().size
	
	data_img.resize(size.x, size.y, Image.INTERPOLATE_NEAREST)
	data_img.fill(Color(0.5, 0.5, 0.5, 1))
	raw_image.set_image(data_img)


func _on_connection_status_changed(connected):
	if !connected:
		_on_size_changed()
